/**
 * ASTStatementList.java
 * 
 * Copyright (c) 2006 David Holroyd
 */

package uk.co.badgersinfoil.metaas.impl;

import org.asdt.core.internal.antlr.AS3Parser;
import uk.co.badgersinfoil.metaas.dom.ASBlock;
import uk.co.badgersinfoil.metaas.dom.ASIfStatement;
import uk.co.badgersinfoil.metaas.dom.ASSwitchStatement;
import uk.co.badgersinfoil.metaas.dom.StatementContainer;
import uk.co.badgersinfoil.metaas.impl.antlr.LinkedListToken;
import uk.co.badgersinfoil.metaas.impl.antlr.LinkedListTree;

/**
 * Implements the behavior of StatementContainer.  May be managing statements
 * for either be a block, or a switch-statement label.
 */
public class ASTStatementList implements ASBlock {

	private LinkedListTree ast;

	public ASTStatementList(LinkedListTree ast) {
		this.ast = ast;
	}

	public void addStmt(String statement) {
		LinkedListTree stmt = AS3FragmentParser.parseStatement(statement);
		stmt.getStopToken().setNext(null);
		addStatement(stmt);
	}

	public void addComment(String text) {
		LinkedListToken comment = TokenBuilder.newSLComment("//" + text);
		String indent = findIndentForComment();
		ast.appendToken(TokenBuilder.newNewline());
		ast.appendToken(TokenBuilder.newWhiteSpace(indent));
		ast.appendToken(comment);
	}

	private String findIndentForComment() {
		LinkedListTree last = ast.getLastChild();
		String indent;
		if (last == null) {
			indent = "\t" + ASTUtils.findIndent(ast);
		} else {
			indent = ASTUtils.findIndent(last);
		}
		return indent;
	}

	public ASIfStatement newIf(String condition) {
		LinkedListTree ifStmt = ASTFactory.newIf(condition);
		addStatement(ifStmt);
		return new ASTASIfStatement(ifStmt);
	}

	public StatementContainer newFor(String init, String condition, String iterate) {
		LinkedListTree forStmt = ASTFactory.newFor(init, condition, iterate);
		addStatement(forStmt);
		LinkedListTree block = appendBlock(forStmt);
		return new ASTStatementList(block);
	}

	public StatementContainer newForIn(String declaration, String expression) {
		LinkedListTree forStmt = ASTFactory.newForIn(declaration, expression);
		LinkedListTree block = appendBlock(forStmt);
		addStatement(forStmt);
		return new ASTStatementList(block);
	}

	public StatementContainer newForEachIn(String declaration, String expression) {
		LinkedListTree forStmt = ASTFactory.newForEachIn(declaration, expression);
		LinkedListTree block = appendBlock(forStmt);
		addStatement(forStmt);
		return new ASTStatementList(block);
	}

	public StatementContainer newWhile(String condition) {
		LinkedListTree whileStmt = ASTFactory.newWhile(condition);
		LinkedListTree block = appendBlock(whileStmt);
		addStatement(whileStmt);
		return new ASTStatementList(block);
	}

	public StatementContainer newDoWhile(String condition) {
		LinkedListTree doWhileStmt = ASTFactory.newDoWhile(condition);
		addStatement(doWhileStmt);
		return new ASTStatementList(findBlock(doWhileStmt));
	}

	public ASSwitchStatement newSwitch(String condition) {
		LinkedListTree switchStmt = ASTFactory.newSwitch(condition);
		addStatement(switchStmt);
		return new ASTASSwitchStatement(findBlock(switchStmt));
	}

	private void addStatement(LinkedListTree stmt) {
		ASTUtils.addChildWithIndentation(ast, stmt);
	}

	public boolean containsCode() {
		return ast.getFirstChild() != null;
	}

	public LinkedListTree getAST() {
		return ast;
	}

	/**
	 * Appends a block node as a child of the given node (preceeding it
	 * with a space token) and returns a reference to the new block.
	 */
	private static LinkedListTree appendBlock(LinkedListTree ast) {
		ast.appendToken(TokenBuilder.newSpace());
		LinkedListTree block = ASTFactory.newBlock();
		ast.addChildWithTokens(block);
		return block;
	}

	/**
	 * Returns the first BLOCK child of the given node
	 */
	private static LinkedListTree findBlock(LinkedListTree ast) {
		return (LinkedListTree)ast.getFirstChildWithType(AS3Parser.BLOCK);
	}
}